/* XAPP052 */

module LFSR (CLK, E, RESET, SEED, LFSR);
parameter NUM_BITS = 32;
input CLK;
input E;
input RESET;
input [(NUM_BITS - 1):0] SEED;
output reg [NUM_BITS:1] LFSR;

reg XNOR;

always @(posedge CLK) begin
	if (!E) begin
		if (RESET) LFSR <= SEED;
	end else begin
		case (NUM_BITS)
			3: XNOR = LFSR[3] ^~ LFSR[2];
			4: XNOR = LFSR[4] ^~ LFSR[3];
			5: XNOR = LFSR[5] ^~ LFSR[3];
			6: XNOR = LFSR[6] ^~ LFSR[5];
			7: XNOR = LFSR[7] ^~ LFSR[6];
			8: XNOR = LFSR[8] ^~ LFSR[6] ^~ LFSR[5] ^~ LFSR[4];
			9: XNOR = LFSR[9] ^~ LFSR[5];
			10: XNOR = LFSR[10] ^~ LFSR[7];
			11: XNOR = LFSR[11] ^~ LFSR[9];
			12: XNOR = LFSR[12] ^~ LFSR[6] ^~ LFSR[4] ^~ LFSR[1];
			13: XNOR = LFSR[13] ^~ LFSR[4] ^~ LFSR[3] ^~ LFSR[1];
			14: XNOR = LFSR[14] ^~ LFSR[5] ^~ LFSR[3] ^~ LFSR[1];
			15: XNOR = LFSR[15] ^~ LFSR[14];
			16: XNOR = LFSR[16] ^~ LFSR[15] ^~ LFSR[13] ^~ LFSR[4];
			17: XNOR = LFSR[17] ^~ LFSR[14];
			18: XNOR = LFSR[18] ^~ LFSR[11];
			19: XNOR = LFSR[19] ^~ LFSR[6] ^~ LFSR[2] ^~ LFSR[1];
			20: XNOR = LFSR[20] ^~ LFSR[17];
			21: XNOR = LFSR[21] ^~ LFSR[19];
			22: XNOR = LFSR[22] ^~ LFSR[21];
			23: XNOR = LFSR[23] ^~ LFSR[18];
			24: XNOR = LFSR[24] ^~ LFSR[23] ^~ LFSR[22] ^~ LFSR[17];
			25: XNOR = LFSR[25] ^~ LFSR[22];
			26: XNOR = LFSR[26] ^~ LFSR[6] ^~ LFSR[2] ^~ LFSR[1];
			27: XNOR = LFSR[27] ^~ LFSR[5] ^~ LFSR[2] ^~ LFSR[1];
			28: XNOR = LFSR[28] ^~ LFSR[25];
			29: XNOR = LFSR[29] ^~ LFSR[27];
			30: XNOR = LFSR[30] ^~ LFSR[6] ^~ LFSR[4] ^~ LFSR[1];
			31: XNOR = LFSR[31] ^~ LFSR[28];
			32: XNOR = LFSR[32] ^~ LFSR[22] ^~ LFSR[2] ^~ LFSR[1];
			64: XNOR = LFSR[64] ^~ LFSR[63] ^~ LFSR[61] ^~ LFSR[60];
		endcase
		LFSR <= { LFSR[(NUM_BITS - 1):1], XNOR };
	end
end

endmodule

/* RC driver, positive logic */

module uint64_drv (CLK, OE, data, row, column);
input CLK;
input OE;
input [63:0] data;
output [7:0] row;
output [7:0] column;

reg [2:0] byte_sel;
reg [7:0] trow;
reg [7:0] tcolumn;
	
always @(posedge CLK) begin
	if (!OE) begin
		trow <= 8'b00000000;
		tcolumn <= 8'b00000000;
		byte_sel <= 3'b000;
	end else begin
		case (byte_sel)
			3'b000: begin
				trow <= 8'b00000001;
				tcolumn <= data[7:0];
			end
			3'b001: begin
				trow <= 8'b00000010;
				tcolumn <= data[15:8];
			end
			3'b010: begin
				trow <= 8'b00000100;
				tcolumn <= data[23:16];
			end
			3'b011: begin
				trow <= 8'b00001000;
				tcolumn <= data[31:24];
			end
			3'b100: begin
				trow <= 8'b00010000;
				tcolumn <= data[39:32];
			end
			3'b101: begin
				trow <= 8'b00100000;
				tcolumn <= data[47:40];
			end
			3'b110: begin
				trow <= 8'b01000000;
				tcolumn <= data[55:48];
			end
			3'b111: begin
				trow <= 8'b10000000;
				tcolumn <= data[63:56];
			end
		endcase
		byte_sel <= byte_sel + 1;
	end
end

assign row = trow;
assign column = tcolumn;

endmodule

/* TOP */
/* we'll be watching the LFSR register */
/* clock @12MHz */

module blinker_8x8 (CLK, rows, columns);
input CLK;
output [7:0] rows;
output [7:0] columns;

localparam NUM_BITS = 64;
localparam LFSR_CLK = 20; /* ~11Hz */
localparam UPDATE_CLK = 13; /* ~680us */

reg [23:0] counter;
reg [NUM_BITS-1:0] LFSR;

LFSR #(NUM_BITS) LFSR_inst(
	.CLK(counter[LFSR_CLK]),
	.E(1'b1),
	.RESET(1'b0),
	.SEED({ NUM_BITS{1'b0} }),
	.LFSR(LFSR),
);

uint64_drv drv(
	.CLK(counter[UPDATE_CLK]),
	.OE(1'b1),
	.data(LFSR),
	.row(rows),
	.column(columns)
);

always @(posedge CLK) begin
	counter <= counter + 1;
end

endmodule

